using System;
using System.Text;
using System.Net;
using System.IO;
using System.Threading;
using System.Collections;
using System.Collections.Specialized;
using System.Diagnostics;

using ICSharpCode.SharpZipLib.GZip;
using ICSharpCode.SharpZipLib.Zip.Compression.Streams;

namespace RSSReader.HTTP
{


	#region WebResponse class 

	/// <summary>
	/// Wrapper for information returned by an HttpWebRequest
	/// </summary>
	public class WebResponse
	{

		#region Private member variables 

		private string				url;
		private string				data;
		private StringDictionary	headers;
		private object				tag;
		private HttpStatusCode		statusCode;

		#endregion 

		#region Constructor 

		private WebResponse() {}

		public WebResponse( System.Net.HttpWebResponse Response, string Url, object Tag )
		{
			
			this.tag = Tag;
			this.url = Url;
			this.data = "";

			using( Response ) 
			{
				ProcessResponse( Response );
			}

		}

		#endregion 

		#region Public properties 

		public HttpStatusCode StatusCode
		{
			get { return statusCode; }
		}

		public string LastModified
		{
			
			get
			{

				if( !headers.ContainsKey( "Last-Modified" ) )
					return DateTime.Now.ToString();
				else
					return headers[ "Last-Modified" ];

			}

		}

		public string ETag
		{
			
			get
			{

				if( !headers.ContainsKey( "eTag" ) )
					return "";
				else
					return headers[ "eTag" ];

			}

		}

		public string Data
		{
			get { return data; }
		}

		public StringDictionary Headers
		{
			get { return headers; }
		}

		public object Tag
		{
			get { return tag; }
		}

		public string Url
		{
			get { return url; }
		}

		#endregion 

		#region Private utility methods 

		private void ProcessResponse( HttpWebResponse response )
		{

			ExtractHeaders( response.Headers );

			statusCode = response.StatusCode;
			if( response.StatusCode != HttpStatusCode.OK )
				return;

			if( response.ContentLength == 0 )
				return;

			Stream dataStream = null;
			if( response.ContentEncoding == "gzip" || response.ContentEncoding == "deflate" )
				dataStream = DecompressStream( response.ContentEncoding, response.GetResponseStream() );
			else
				dataStream = response.GetResponseStream();

			using( StreamReader reader = new StreamReader( dataStream ) )
			{
				data = reader.ReadToEnd();
			}

		}

		private Stream DecompressStream( string encoding, Stream stream )
		{

			Stream compressed = null; 

			if( encoding == "deflate" ) 
				compressed = new InflaterInputStream( stream );
			else if( encoding == "gzip" ) 
				compressed = new GZipInputStream( stream );
			
			if( compressed == null ) 
				return stream;
			
			MemoryStream decompressed = new MemoryStream();

			int size = 2048;
			byte[] buffer = new byte[2048];

			while( size > 0 ) 
			{
			
				size = compressed.Read( buffer, 0, size );
				if( size > 0 ) 
					decompressed.Write( buffer, 0, size );
				else 
					break;
			
			}

			decompressed.Seek( 0, SeekOrigin.Begin );
			return decompressed;

		}

		private void ExtractHeaders( WebHeaderCollection responseHeaders )
		{

			headers = new CaseInsensitiveStringDictionary();
			foreach( string key in responseHeaders.AllKeys )
			{

				string headerValue = responseHeaders[ key ];
				headers.Add( key, headerValue );

			}

		}

		#endregion 

	}

	#endregion 

	/// <summary>
	/// Helper methods for HTTP data retrieval
	/// </summary>
	public class WebRequest
	{

		#region Private member variables 

		private	string			url;
		private string			etag;
		private DateTime		lastModified;
		private object			tag;
		private IWebProxy		proxy;
		private ICredentials	credentials;

		#endregion 

		#region Constructor 

		public WebRequest() 
		{ 
			
			url = "";
			etag = "";
			lastModified = new DateTime( 1970, 1, 1 );
			tag = null;
			proxy = GlobalProxySelection.GetEmptyWebProxy();
			credentials = null;

		}

		#endregion 

		#region Public properties 

		public string Url
		{
			get { return url; }
			set { url = value; }
		}

		public string eTag
		{
			get { return etag; }
			set { etag = value; }
		}

		public DateTime LastModified
		{
			get { return lastModified; }
			set { lastModified = value; }
		}

		public IWebProxy Proxy
		{
			get { return proxy; }
			set { proxy = value; }
		}

		public ICredentials Credentials
		{
			get { return credentials; }
			set { credentials = value; }
		}

		#endregion 

		#region GetResponse() main method 

		public WebResponse GetResponse()
		{
			return GetResponse( true, 1 );
		}

		public WebResponse GetResponse( bool AllowCompression, int Attempt )
		{

			if( url == "" || Attempt >= 3 )
				throw new Exception( "You must provide a value for the Url property" );

			HttpWebRequest request    = (HttpWebRequest) System.Net.WebRequest.Create( url );
			request.Timeout           = (int) TimeSpan.FromSeconds( 30 ).TotalMilliseconds;
			request.UserAgent         = "SharpTools RSS Reader V0.3A"; 
			request.AllowAutoRedirect = false;
			request.IfModifiedSince   = lastModified;
			request.Referer			  = "http://www.ms-inc.net";
			
			request.Headers.Add( "ETag", etag );

			if( AllowCompression )
				request.Headers.Add( "Accept-Encoding", "gzip, deflate" );

			if( proxy == null ) 
			{
				request.KeepAlive = false;	
				request.Proxy = GlobalProxySelection.GetEmptyWebProxy();
				request.Proxy.Credentials = CredentialCache.DefaultCredentials;
			}

			if( credentials != null )
			{

				// Required for authentication to succeed
				request.KeepAlive = true;				
	
				request.ProtocolVersion = HttpVersion.Version11;	
				request.Credentials = credentials;

			}
			
			try
			{
				return new WebResponse( (HttpWebResponse) request.GetResponse(), this.url, tag );
			}
			catch( ICSharpCode.SharpZipLib.SharpZipBaseException )
			{
				SharpTools.Globals.OutputWindow.WriteLine( "Invalid HttpCompress header received from: " + url + "\r\n\tRetrying without compression" );
				return GetResponse( false, Attempt + 1 );
			}
			catch( Exception err )
			{
				SharpTools.Globals.OutputWindow.WriteLine( "Error retrieving feed data (" + err.Message + ") from " + url );
				return null;
			}

		}

		#endregion 
		
	}

}
