using System;
using System.Text;
using System.Collections;
using System.ComponentModel;
using System.Xml;

using RSSReader.RSS;

namespace RSSReader.OPML
{

	#region OPMLNode class 

	/// <summary>
	/// Represents an OPML element in a subscription file
	/// </summary>
	[DefaultProperty("Title")]
	public class OPMLNode : IComparable
	{

		#region Private member variables 

		private	OPMLNodeCollection collection;

		protected	OPMLNode	parent;
		protected	string		title;
		protected	string		description;

		#endregion 

		#region Constructor 

		protected OPMLNode()
		{

			parent = null;

			title = "";
			description = "";

		}

		protected OPMLNode( XmlNode Element )
		{

			if( Element.Name != "outline" )
				throw new Exception( "Element is not a valid OPML element: " + Element.Name );

			title = XmlHelper.GetAttribute( Element, "title", "" );
			description = XmlHelper.GetAttribute( Element, "description", "" );
			parent = null;

		}

		#endregion 

		#region Public properties 

		[Category("Feed"),Description("The name of the RSS Feed")]
		public string Title
		{
			get { return title; }
			set { title = value; }
		}

		[Category("Feed"),Description("A brief description of the feed's typical content")]
		public string Description 
		{
			get { return description; }
			set { description = value; }
		}

		[Browsable(false)]
		public string Path
		{

			get
			{

				StringBuilder buffer = new StringBuilder();

				OPMLNode loop = this;
				while( loop != null )
				{

					if( buffer.Length > 0 )
						buffer.Insert( 0, "/" );

					buffer.Insert( 0, loop.Title );

					loop = loop.parent;

				}

				return buffer.ToString();

			}

		}

		#endregion 

		#region Internal properties and methods 

		internal OPMLNode Parent
		{
			get { return parent; }
			set { parent = value; }
		}

		internal OPMLNodeCollection Collection
		{
			get { return collection; }
			set { collection = value; }
		}

		#endregion 

		#region Public methods 

		public virtual void Delete()
		{
			Delete( false );
		}

		public virtual void Delete( bool DeleteCachedData )
		{

			if( this.collection != null )
				this.collection.Remove( this );

		}

		#endregion 

		#region Serialization methods 

		public virtual void WriteToXml( XmlTextWriter writer )
		{
			throw new Exception( "Base virtual method called.  This method must be overridden in an inherited class." );
		}

		#endregion 

		#region Static Class Factory methods 

		public static OPMLNode FromXml( XmlNode Element )
		{

			if( Element.Attributes["xmlUrl"] == null )
				return new OPMLCategoryNode( Element );
			else
				return new OPMLItemNode( Element );

		}

		#endregion 
 
		#region IComparable Members 

		internal string GetCompareString()
		{
			
			string compareCode = "";
			if( this is OPMLCategoryNode )
				compareCode = "1";
			else
				compareCode = "2";

			return compareCode + this.title;

		}

		public int CompareTo(object obj)
		{
			return GetCompareString().CompareTo( ((OPMLNode) obj).GetCompareString() );
		}

		#endregion

	}

	#endregion 

	#region OPMLCategoryNode class 

	/// <summary>
	/// Represents an OPML node which acts as a container, or
	/// category, for other OPML nodes
	/// </summary>
	public class OPMLCategoryNode : OPMLNode
	{

		#region Private member variables 

		protected OPMLNodeCollection nodes;

		#endregion 

		#region Constructor 

		internal OPMLCategoryNode() : base()
		{
			nodes = new OPMLNodeCollection();
			nodes.InsertComplete += new RSSReader.OPML.OPMLNodeCollection.OPMLNodeCollectionChangedEventHandler(nodes_InsertComplete);
		}

		internal OPMLCategoryNode( XmlNode Element ) : base( Element )
		{

			nodes = new OPMLNodeCollection();
			nodes.InsertComplete += new RSSReader.OPML.OPMLNodeCollection.OPMLNodeCollectionChangedEventHandler(nodes_InsertComplete);

			foreach( XmlNode child in Element.ChildNodes )
			{

				if( child.Name == "outline" )
					nodes.Add( OPMLNode.FromXml( child ) );

			}

			nodes.Sort();

		}

		#endregion 

		#region Public properties 

		[Browsable(false)]
		public OPMLNodeCollection Nodes
		{
			get { return nodes; }
		}

		#endregion 

		#region Overridden methods 

		public override void Delete( bool DeleteCachedData )
		{

			if( DeleteCachedData )
			{

				foreach( OPMLNode child in this.nodes.Clone() )
					child.Delete( DeleteCachedData );
			
			}

			this.Collection.Remove( this );

		}


		#endregion 

		#region Serialization methods 

		public override void WriteToXml(XmlTextWriter writer)
		{

			nodes.Sort();
			
			writer.WriteStartElement( "outline" );

			writer.WriteAttributeString( "title", title );
			writer.WriteAttributeString( "description", description );

			foreach( OPMLNode child in nodes )
			{
				child.WriteToXml( writer );
			}

			writer.WriteEndElement();

		}

		#endregion

		#region Nodes collection event handlers 

		private void nodes_InsertComplete(object sender, OPMLNode value)
		{
			value.Parent = this;
			value.Collection = nodes;
		}

		#endregion 

	}

	#endregion 

	#region OPMLItemNode class 

	/// <summary>
	/// Represents an OPML node which is a subscription
	/// </summary>
	public class OPMLItemNode : OPMLNode
	{

		#region Private member variables 

		protected string xmlUrl;
		protected string htmlUrl;

		#endregion 

		#region Constructor 

		internal OPMLItemNode() : base()
		{
		}

		internal OPMLItemNode( XmlNode Element ) : base( Element )
		{

//			if( XmlHelper.GetAttribute( Element, "type", "" ) != "rss" )
//				throw new Exception( "Element is not a valid OPML element: Missing or invalid 'type' attribute" );

			xmlUrl = XmlHelper.GetAttribute( Element, "xmlUrl", "" );
			htmlUrl = XmlHelper.GetAttribute( Element, "htmlUrl", "" );
		
		}

		#endregion 

		#region Public properties 

		[Category("URL"),Description("The URL from which to retrieve the RSS feed data.")]
		public string XmlUrl
		{
			get { return xmlUrl; }
			set { xmlUrl = value; }
		}

		[Category("URL"),Description("The address of the website hosting this feed.")]
		public string HtmlUrl
		{
			get { return htmlUrl; }
			set { htmlUrl = value; }
		}

		#endregion 

		#region Overridden methods

		public override void Delete( bool DeleteCachedData )
		{
			
			if( DeleteCachedData )
				FeedManager.DeleteFeedFromCache( this.xmlUrl );

			this.Collection.Remove( this );

		}


		#endregion 

		#region Serialization methods 

		public override void WriteToXml(XmlTextWriter writer)
		{
			
			writer.WriteStartElement( "outline" );

			writer.WriteAttributeString( "type", "rss" );
			writer.WriteAttributeString( "title", title );
			writer.WriteAttributeString( "description", description );
			writer.WriteAttributeString( "xmlUrl", xmlUrl );
			writer.WriteAttributeString( "htmlUrl", htmlUrl );

			writer.WriteEndElement();

		}

		#endregion 

	}

	#endregion 

}
