using System;
using System.Collections;
using System.Xml;

namespace RSSReader.RSS
{

	/// <summary>
	/// Represents the information parsed from an {item} element in an RSS feed
	/// </summary>
	public class RSSItem : IComparable
	{

		#region Private member variables 

		private XmlNamespaceManager ns;
		private XmlNode				sourceNode;
		private ItemSortField		sortField = ItemSortField.Date;
		private ItemSortOrder		sortOrder = ItemSortOrder.Descending;

		private RSSFeed		parent;
		private RSSFeed		comments;
		private string		title = "";
		private string		link = "";
		private string		guid = "";
		private string		description = "";
		private string		subject = "";
		private string		creator = "";
		private DateTime	date = DateTime.Now;
		private string		commentPostUrl = "";
		private string		commentFeedUrl = "";
		private string		sourceUrl = "";
		private string		sourceTitle = "";

		private bool		markedAsRead = false;
		private bool		markedForFollowUp = false;
		private bool		isDeleted = false;

		#endregion 

		#region Constructor 

		private RSSItem() {}

		internal RSSItem( XmlNode Element )
		{
			ReadFromXml( Element );
		}

		#endregion 

		#region Public properties 

		/// <summary>
		/// Gets/Sets the headline text that will be displayed
		/// </summary>
		public string Title
		{
			get { return title; }
			set { title = value; }
		}

		/// <summary>
		/// Gets/Sets the url to the original post
		/// </summary>
		public string Link
		{
			get { return link; }
			set { link = value; }
		}

		/// <summary>
		/// Gets/Sets a unique identifier for this RSSItem
		/// </summary>
		public string Guid
		{
			get { return guid; }
			set { guid = value; }
		}

		/// <summary>
		/// Gets/Sets the RSSItem message body
		/// </summary>
		public string Description 
		{
			get { return description; }
			set { description = value; }
		}

		/// <summary>
		/// Gets/Sets a value that indicates the author of the original post
		/// </summary>
		public string Creator
		{
			get { return creator; }
			set { creator = value; }
		}

		/// <summary>
		/// Gets/Sets the subject line of the post
		/// </summary>
		public string Subject
		{
			get { return subject; }
			set { subject = value; }
		}
 
		/// <summary>
		/// Gets/Sets the date on which the post was published
		/// </summary>
		public DateTime DatePublished
		{
			get { return date; }
			set { date = value; }
		}

		/// <summary>
		/// Gets/Sets the url to which the user should navigate to
		/// post comments and replies to the post
		/// </summary>
		public string CommentPostUrl
		{
			get { return commentPostUrl; }
			set { commentPostUrl = value; }
		}

		/// <summary>
		/// Gets/Sets the url to which the user should navigate to
		/// view the comments/replies to this post
		/// </summary>
		public string CommentFeedUrl
		{
			get { return commentFeedUrl; }
			set { commentFeedUrl = value; }
		}

		/// <summary>
		/// Gets/Sets a value indicating whether the user has read this post
		/// </summary>
		public bool MarkedAsRead
		{
			get { return markedAsRead; }
			set { markedAsRead = value; }
		}

		/// <summary>
		/// Gets/Sets a value indicating whether the user has marked this
		/// post for follow-up
		/// </summary>
		public bool MarkedForFollowUp
		{
			get { return markedForFollowUp; }
			set { markedForFollowUp = value; }
		}

		/// <summary>
		/// Gets an RSSFeed object containing all replies to this post
		/// </summary>
		public RSSFeed Comments 
		{
			
			get
			{

				if( comments == null && commentFeedUrl != "" )
					comments = FeedManager.GetCachedFeed( commentFeedUrl );

				return comments;

			}

		}

		#endregion 

		#region Internal methods 

		/// <summary>
		/// Gets a value indicating whether this post has been deleted
		/// </summary>
		internal bool Deleted
		{

			get { return isDeleted; }

			set 
			{ 
			
				if( isDeleted == true && value == false )
					throw new Exception( "Cannot set Deleted property to FALSE, there is not enough information persisted to 'Undelete' an item" );
				else
				{

					if( value == false )
						return;

				}

				isDeleted = true; 

				FeedManager.DeleteFeedFromCache( this.commentFeedUrl );
			
			}

		}

		internal void RetrieveComments()
		{

			if( commentFeedUrl != "" )
			{

				comments = FeedManager.RetrieveFeed( commentFeedUrl );

				if( comments != null )
					FeedManager.SaveFeed( comments );
			}

		}

		#endregion 

		#region Internal properties 

		internal RSSFeed Parent
		{
			get { return parent; }
			set { parent = value; }
		}

		internal ItemSortField SortField
		{
			
			get { return sortField; }
			
			set
			{
				
				sortField = value; 

				if( comments != null )
				{

					foreach( RSSItem comment in comments.Items )
					{
						comment.SortField = value;
					}

				}

			}

		}

		internal ItemSortOrder SortOrder
		{

			get { return sortOrder; }
			
			set 
			{ 
				
				sortOrder = value; 

				if( comments != null )
				{
				
					foreach( RSSItem comment in comments.Items )
					{
						comment.SortOrder = value;
					}

				}

			}
		}

		#endregion 

		#region Serialization  

		#region ReadFromXml and support methods 

		private void ReadFromXml( XmlNode Element )
		{

			// Set up commonly used RSS namespaces
			ns = new XmlNamespaceManager( Element.OwnerDocument.NameTable );
			ns.AddNamespace( "dc", "http://purl.org/dc/elements/1.1/" );
			ns.AddNamespace( "content", "http://purl.org/rss/1.0/modules/content/" );
			ns.AddNamespace( "wfw", "http://wellformedweb.org/CommentAPI/" );
			ns.AddNamespace( "rss", Element.NamespaceURI );

			sourceNode = Element;

			// Read the only properties that will be set if this post has 
			// been deleted.
			guid = GetValue( "", "rss:guid", "rss:link" );
			isDeleted = GetValue( false, "Deleted" );
			if( isDeleted )
				return;

			// Read common and simple properties 
			title = GetValue( "", "rss:title" );
			link = GetValue( "", "rss:link", "rss:guid" );
			subject = GetValue( "", "dc:subject", "rss:category", "subject" );
			description = GetValueXml( "", "content:encoded", "rss:description", "dc:descripton" );
			commentPostUrl = GetValue( "", "rss:comments", "wfw:comment", "comments" );
			commentFeedUrl = GetValue( "", "wfw:commentRss","commentRss" );
			markedAsRead = GetValue( false, "MarkedAsRead" );
			markedForFollowUp = GetValue( false, "MarkedForFollowUp" );

			if( description.ToLower().StartsWith("<[cdata[") )
			{
				description = description.Substring( 8 );
				description = description.Substring( 0, description.Length - 3 );
			}

			// Unbelievable, but true: Some feeds have no title whatsoever,
			// so we just use the first 70 characters of the description.
			// I thought <title> was required since 0.9?????
			if( title == "" )
			{

				string temp = StripHtml.Strip( System.Web.HttpUtility.HtmlDecode( description ) );
				
				if( temp.Length > 70 )
					title = temp.Substring( 0, 70 ) + "...";
				else
					title = temp.Substring( 0, Math.Min( 70, temp.Length ) );

			}

			// Read properties that require more processing
			date = GetDate();
			creator = GetCreator();

			#region Determine whether this item was consolidated from another feed

			// For example, some blogs like http://blogs.msdn.com/MainFeed.aspx 
			// consolidate all available feeds into one main feed.
			XmlNode source = Element.SelectSingleNode( "source" );
			if( source == null )
			{
				sourceUrl = "";
				sourceTitle = "";
			}
			else
			{
				
				if( source.Attributes["url"] != null )
					sourceUrl = source.Attributes["url"].InnerText;

				sourceTitle = source.InnerText;

			}

			#endregion 
		
		}

		private string GetCreator()
		{

			string author = GetValue( "", "dc:creator", "rss:creator", "creator" );
			if( author != "" )
				return author;

			XmlNode node = sourceNode;
			while( node != null && node.LocalName != "channel" )
				node = node.ParentNode;

			if( node == null )
				return "";
			else
				return GetValue( node, "", "dc:creator", "rss:creator", "creator" );

		}

		private DateTime GetDate()
		{

			try
			{

				string dateText = GetValue( "", "dc:date", "date" );
				if( dateText != "" )
					return XmlConvert.ToDateTime( dateText );

				dateText = GetValue( "", "rss:pubDate" );
				if( dateText != "" )
					return DateTime.Parse( dateText );

			}
			catch( FormatException )
			{
			}

			return DateTime.Today;

			
		}

		private string GetValueXml( XmlNode Node, string DefaultValue, params string[] ElementNames )
		{

			if( ElementNames == null || ElementNames.Length == 0 )
				throw new ArgumentException( "You must provide an Element Name or a list of Element Names", "ElementNames" );

			foreach( string elementName in ElementNames )
			{

				XmlNode node = Node.SelectSingleNode( elementName, ns );
				if( node != null )
					return node.InnerXml;

			}

			return DefaultValue;

		}

		private bool GetValueXml( bool DefaultValue, params string[] ElementNames )
		{
			
			string boolValue = GetValueXml( DefaultValue.ToString(), ElementNames );
			return bool.Parse( boolValue );

		}

		private string GetValueXml( string DefaultValue, params string[] ElementNames )
		{
			return GetValueXml( sourceNode, DefaultValue, ElementNames );
		}

		private string GetValue( XmlNode Node, string DefaultValue, params string[] ElementNames )
		{

			if( ElementNames == null || ElementNames.Length == 0 )
				throw new ArgumentException( "You must provide an Element Name or a list of Element Names", "ElementNames" );

			foreach( string elementName in ElementNames )
			{

				XmlNode node = Node.SelectSingleNode( elementName, ns );
				if( node != null )
					return node.InnerText;

			}

			return DefaultValue;

		}

		private bool GetValue( bool DefaultValue, params string[] ElementNames )
		{
			
			string boolValue = GetValue( DefaultValue.ToString(), ElementNames );
			return bool.Parse( boolValue );

		}

		private string GetValue( string DefaultValue, params string[] ElementNames )
		{
			return GetValue( sourceNode, DefaultValue, ElementNames );
		}

		#endregion 

		#region WriteToXml and support methods 

		public void WriteToXml( XmlTextWriter writer )
		{

			writer.WriteStartElement( "item" );

			XmlHelper.WriteSimpleElement( writer, "guid", guid );
			XmlHelper.WriteSimpleElement( writer, "Deleted", isDeleted );
			
			if( !isDeleted )
			{

				XmlHelper.WriteSimpleElement( writer, "title", title );
				XmlHelper.WriteSimpleElement( writer, "link", link );
				XmlHelper.WriteSimpleElement( writer, "subject", subject );
				XmlHelper.WriteSimpleElement( writer, "creator", creator );
				XmlHelper.WriteSimpleElement( writer, "pubDate", date.ToString() );
				XmlHelper.WriteSimpleElement( writer, "comments", commentPostUrl );
				XmlHelper.WriteSimpleElement( writer, "commentRss", commentFeedUrl );
				XmlHelper.WriteSimpleElement( writer, "MarkedAsRead", markedAsRead );
				XmlHelper.WriteSimpleElement( writer, "MarkedForFollowUp", markedForFollowUp );

				if( sourceUrl != "" || sourceTitle != "" )
				{

					writer.WriteStartElement( "source" );
				
					if( sourceUrl != "" )
					{
						writer.WriteStartAttribute( "", "url", "" );
						writer.WriteString( sourceUrl );
						writer.WriteEndAttribute();
					}

					if( sourceTitle != "" )
						writer.WriteString( sourceTitle );

					writer.WriteEndElement();

				}

				writer.WriteStartElement( "description");
				writer.WriteRaw( description );
				writer.WriteEndElement();

			}

			writer.WriteEndElement();

		}

		#endregion 

		#endregion 

		#region IComparable Members 

		public int CompareTo( object obj )
		{

			// If this item has been deleted, we *always* want it to come
			// last in the list.
			if( this.isDeleted )
				return 1;

			int returnValue = 0;

			RSSItem compareObj = (RSSItem) obj;
			switch( sortField )
			{

				case ItemSortField.MarkedAsRead:
					if( this.MarkedAsRead == compareObj.MarkedAsRead )
						returnValue = new DateColumnComparer().Compare( date, compareObj.date );
					else
						returnValue = this.MarkedAsRead.CompareTo( compareObj.MarkedAsRead );
					break;

				case ItemSortField.Title:
					returnValue = new System.Collections.CaseInsensitiveComparer().Compare( title, compareObj.title );
					break;

				case ItemSortField.Creator:
					returnValue = new System.Collections.CaseInsensitiveComparer().Compare( creator, compareObj.creator );
					break;

				case ItemSortField.Subject:
					returnValue = new System.Collections.CaseInsensitiveComparer().Compare( subject, compareObj.subject );
					break;

				case ItemSortField.Date:
					returnValue = new DateColumnComparer().Compare( date, compareObj.date );
					break;

				default: 
					throw new Exception( "Unhandled sort type: " + sortField.ToString() );

			}

			// Return the opposite value for Descending order
			if( sortOrder == ItemSortOrder.Descending )
				returnValue *= -1;

			return returnValue;

		}

		#endregion

	}

	#region Enumerations for custom sorting 

	/// <summary>
	/// Specified whether the RSSItem object will be sorted in 
	/// Ascending or Descending order
	/// </summary>
	public enum ItemSortOrder
	{
		Ascending,
		Descending
	}

	/// <summary>
	/// Specifies which RSSItem property will be used in the custom
	/// sort.
	/// </summary>
	public enum ItemSortField
	{
		Title,
		Date,
		Creator,
		Subject,
		MarkedAsRead
	}

	#endregion 

}
