/////////////////////////////////////////////////////////////////////////
// Copyright Morrison Schwartz, Inc 2003-2004, All rights reserved     //
//---------------------------------------------------------------------//
// This product is protected by US Copyright laws and international    //
// treaties.  For more information, see the End User License Agreement //
// in the application installation folder.                             //
//---------------------------------------------------------------------//
// Visit us on the web at: http://www.ms-inc.net                       //
/////////////////////////////////////////////////////////////////////////
using System;
using System.IO;
using System.Collections;
using System.Text;
using System.Xml;
using System.Xml.XPath;
using System.Xml.Schema;
using System.Xml.Xsl;
using System.Data;  

#region SchemaInfo private helper class

/// <summary>
/// Holds information to create a Schema cache for validating XML documents
/// </summary>
class SchemaInfo
{

	private string _namespace = "";
	public string Namespace 
	{
		get { return _namespace; }
		set { _namespace = value; }
	}

	private string _url = "";
	public string URL
	{
		get { return _url; }
		set { _url = value; }
	}

	public SchemaInfo( string Namespace, string URL )
	{
		this._namespace = Namespace;
		this._url = URL;
	}

}
#endregion 

#region XmlValidator class
public class XmlValidator
{

	private XmlTextReader xmlReader = null;
	private XmlValidatingReader xmlValidator = null;

	#region Property methods 

	private ArrayList schemas = new ArrayList();

	private int errorLine = 0;
	public int ErrorLine
	{
		get { return errorLine; }
	}

	private int errorPosition = 0;
	public int ErrorPosition
	{
		get { return errorPosition; }
	}

	private string errorMessage = "";
	public string ErrorMessage
	{
		get { return errorMessage; }
	}

	#endregion 

	#region Default Constructor (stub)
	public XmlValidator()
	{
		// @@@ Stub 
	}
	#endregion

	/// <summary>
	/// Used to indicate the location of a schema file for validation.  Use this function
	/// when the schema file exists in a location other than that indicated within the
	/// xml document (cached schemas).
	/// </summary>
	/// <param name="ns">The namespace that is used for the schema</param>
	/// <param name="url">The URI of the schema file</param>
	public void AddSchema( string ns, string url )
	{
		schemas.Add( new SchemaInfo( ns, url ) );
	}

	/// <summary>
	/// Validates the XML document, checking well-formedness and validating against
	/// either DTD, XDR, or XSD schema.
	/// </summary>
	/// <param name="xmlText">The Xml Text to validate</param>
	/// <param name="validationType">The type of validation to perform.</param>
	/// <returns></returns>
	public bool ValidateXml( System.IO.TextReader xmlText, System.Xml.ValidationType validationType ) 
	{

		try
		{

			// Reset error information 
			errorLine = -1;
			errorPosition = -1;

			// Instantiate and initialize objects necessary for validation 
			xmlReader = new XmlTextReader( xmlText );
			xmlValidator = new XmlValidatingReader( xmlReader );
			xmlValidator.ValidationType = validationType;
			xmlValidator.ValidationEventHandler += new ValidationEventHandler( this.ValidationError );

			// Add any schemas cached in our Schemas collection
			foreach( SchemaInfo schema in schemas )
			{
				xmlValidator.Schemas.Add( schema.Namespace, schema.URL );
			}

			// Perform actual validation 
			while( xmlValidator.Read() && errorLine == -1 ) ;

			// Close the validation stream 
			xmlValidator.Close();

		}
		catch( System.Exception err )
		{
			throw new Exception( "Error attempting to validate XML document", err );
		}

		return (errorLine == -1);

	}

	#region Overloads for ValidateXml()

	public bool ValidateXml( System.IO.TextReader xmlText ) 
	{
		return ValidateXml( xmlText, System.Xml.ValidationType.Auto );
	}

	public bool ValidateXml( string xmlText, System.Xml.ValidationType validationType ) 
	{
		return ValidateXml( new StringReader( xmlText ), validationType );
	}

	public bool ValidateXml( string xmlText ) 
	{
		return ValidateXml( xmlText, System.Xml.ValidationType.Auto );
	}

	#endregion 

	private void ValidationError(Object sender, ValidationEventArgs args)
	{

		// check the severity of the error
		if( Convert.ToInt32(args.Severity) != 1)
			return;

		// Store the line number, position, and error message
		errorLine = xmlReader.LineNumber;
		errorPosition = xmlReader.LinePosition;
		errorMessage = args.Message;

	}
    
}
#endregion

#region XmlHelper class 

/// <summary>
/// Summary description for XmlHelper.
/// </summary>
public sealed class XmlHelper
{

	// Setting default constructor to Private forces use of static methods
	private XmlHelper() {}

	#region Public utility methods

	/// <summary>
	/// Creates a new XmlAttribute object with the specified Name and Value.
	/// Intended to make blocks of code which create a large number of
	/// XmlAttribute objects more readable.
	/// </summary>
	public static XmlAttribute CreateAttribute( XmlDocument Doc, string Name, string Value )
	{

		XmlAttribute attribute = Doc.CreateAttribute( Name );
		attribute.Value = Value;

		return attribute;

	}

	/// <summary>
	/// Retrieves either the named Attribute's Value property or, if the named Attribute
	/// does not exist, returns the DefaultValue value.
	/// </summary>
	public static string GetAttribute( XmlNode Element, string Attribute, string DefaultValue )
	{

		if( Element.Attributes[ Attribute ] != null )
			return Element.Attributes[ Attribute ].Value;
		else
			return DefaultValue;

	}

	/// <summary>
	/// Formats the xml content in the XmlDocument object to make it 
	/// more human-readable.
	/// </summary>
	public static string FormatXml( XmlDocument Document )
	{

		StringBuilder buffer = new StringBuilder();
		XmlTextWriter writer = CreateXmlWriter( new StringWriter( buffer ) );
		Document.WriteContentTo( writer );

		return buffer.ToString();

	}

	public static XmlValidator CreateValidator()
	{
		return new XmlValidator();
	}

	public static XmlTextWriter CreateXmlWriter( System.IO.TextWriter BaseStream )
	{

		// Instantiate an XmlTextWriter to contain the results of the transformation.
		// Formatting is enabled for readability to ease debugging during the initial 
		// development phase
		XmlTextWriter xmlWriter = new XmlTextWriter( BaseStream );
		xmlWriter.Formatting = System.Xml.Formatting.Indented;
		xmlWriter.Indentation = 4;
		xmlWriter.IndentChar = ' ';

		return xmlWriter;

	}

	/// <summary>
	/// Creates an XmlTextReader object from a string.  This is used primarily
	/// to distinguish parameter signatures when calling RenderXML()
	/// </summary>
	/// <param name="XmlText">A valid, well-formed XML text document</param>
	/// <returns></returns>
	public static System.Xml.XmlTextReader StringToXmlTextReader(string XmlText)
	{

		try 
		{

			System.IO.StringReader source = new System.IO.StringReader(XmlText);
			System.Xml.XmlTextReader result = new System.Xml.XmlTextReader(source);

			return result;

		}
		catch( System.Exception err )
		{
			throw( err );
		}

	}

	/// <summary>
	/// Utility function for converting object data into text that will be
	/// stored in the Xml output.  Accepts format specifiers for Date and Number
	/// types, for performing specific conversion when necessary.
	/// </summary>
	public static string FormatAsXml( object value, string dateFormat, string numberFormat )
	{

		// Perform default string conversion 
		string result = value.ToString();

		if( value is DateTime )
		{
			DateTime curDate = (DateTime) value;
			result = curDate.ToString( dateFormat );
		}

		if( value is Decimal )
		{
			Decimal curDecimal = (Decimal) value;
			result = curDecimal.ToString( numberFormat );
		}

		if( value is Double )
		{
			Double curDouble = (Double) value;
			result = curDouble.ToString( numberFormat );
		}

		if( value is Single )
		{
			Single curSingle = (Single) value;
			result = curSingle.ToString( numberFormat );
		}

		return result.Trim();

	}

	public static void WriteSimpleElement( XmlTextWriter writer, string elementName, object value, string dateFormat, string numberFormat )
	{
		writer.WriteStartElement( elementName );
		writer.WriteString( FormatAsXml( value, dateFormat, numberFormat ) );
		writer.WriteEndElement();
	}

	public static void WriteCDATAElement( XmlTextWriter writer, string elementName, string value )
	{
		writer.WriteStartElement( elementName );
		writer.WriteCData( value );
		writer.WriteEndElement();
	}

	public static void WriteSimpleElement( XmlTextWriter writer, string elementName, object value )
	{
		WriteSimpleElement( writer, elementName, value, "d", "n" );
	}

	#endregion 

	#region Render XML data through XSL stylesheets

	private static string LoadTextFile( string Filename )
	{

		using( StreamReader reader = File.OpenText( Filename ) )
		{
			return reader.ReadToEnd();
		}

	}

	/// <summary>
	/// Renders an XML file to HTML (or XML) using XSLT.
	/// </summary>
	/// <param name="Data">A TextReader or XmlTextReader containing valid, well-formed XML</param>
	/// <param name="TransformFile">The path (relative or absolute) to a valid XSLT file</param>
	/// <param name="Parameters">Arguments to be passed to the XSLT file during the transformation process</param>
	/// <returns>When successful, will return the resulting HTML (or XML) data.  When not successful, will return null.</returns>
	public static string RenderXML(System.IO.TextReader Data, string TransformFile, XsltArgumentList Parameters)
	{

		try
		{

			// Instantiate an XmlTextWriter to contain the results of the transformation.
			System.IO.StringWriter buffer = new System.IO.StringWriter();
			XmlTextWriter xmlWriter = CreateXmlWriter( buffer );

			// Instantiate an XPathDocument that will be passed to the XslTransform object
			XPathDocument xdoc = new XPathDocument( Data );

			// Instantiate and load the XSLT file
			XslTransform xslt = new XslTransform();
			//xslt.Load( TransformFile );
			xslt.Load( new XmlTextReader( new StringReader( LoadTextFile( TransformFile ) ) ) );

			// Perform the XSLT transformation.  The results are stored in the XmlTextWriter instantiated earlier.
			// If any parameters were passed, include them in the transformation
			if( Parameters != null )
				xslt.Transform( xdoc, Parameters, xmlWriter );
			else
				xslt.Transform( xdoc, null, xmlWriter );

			// Return the transformed data.
			return buffer.ToString();

		}
		catch( System.Exception err )
		{
			throw( err );
		}

	}

	/// <summary>
	/// Renders an XML file to HTML (or XML) using XSLT.
	/// </summary>
	/// <param name="Data">A TextReader or XmlTextReader containing valid, well-formed XML</param>
	/// <param name="TransformFile">The path (relative or absolute) to a valid XSLT file</param>
	/// <param name="Parameters">Arguments to be passed to the XSLT file during the transformation process</param>
	/// <returns>When successful, will return the resulting HTML (or XML) data.  When not successful, will return null.</returns>
	public static string RenderXML(System.IO.TextReader Data, System.IO.TextReader TransformFile, XsltArgumentList Parameters)
	{

		try
		{

			// Instantiate an XmlTextWriter to contain the results of the transformation.
			System.IO.StringWriter buffer = new System.IO.StringWriter();
			XmlTextWriter xmlWriter = CreateXmlWriter( buffer );

			// Instantiate an XPathDocument that will be passed to the XslTransform object
			XPathDocument xdoc = new XPathDocument( Data );

			// Instantiate and load the XSLT file
			XslTransform xslt = new XslTransform();
			xslt.Load( new XmlTextReader( TransformFile ) ); 

			// Perform the XSLT transformation.  The results are stored in the XmlTextWriter instantiated earlier.
			// If any parameters were passed, include them in the transformation
			if( Parameters != null )
				xslt.Transform( xdoc, Parameters, xmlWriter );
			else
				xslt.Transform( xdoc, null, xmlWriter );

			// Return the transformed data.
			return buffer.ToString();

		}
		catch( System.Exception err )
		{
			throw( err );
		}

	}

	/// <summary>
	/// Renders an XML file to HTML (or XML) using XSLT.
	/// </summary>
	/// <param name="Data">The path (relative or absolute) to a file containing valid, well-formed XML</param>
	/// <param name="TransformFile">The path (relative or absolute) to a valid XSLT file</param>
	/// <param name="Parameters">Arguments to be passed to the XSLT file during the transformation process</param>
	/// <returns>When successful, will return the resulting HTML (or XML) data.  When not successful, will return null.</returns>
	public static string RenderXML(string DataFile, string TransformFile, XsltArgumentList Parameters)
	{

		try
		{
        
			// Instantiate an XmlTextWriter to contain the results of the transformation.
			System.IO.StringWriter buffer = new System.IO.StringWriter();
			XmlTextWriter xmlWriter = CreateXmlWriter( buffer );

			// Instantiate an XPathDocument that will be passed to the XslTransform object
			XPathDocument xdoc = new XPathDocument( DataFile );
            
			// Instantiate and load the XSLT file
			XslTransform xslt = new XslTransform();
			xslt.Load( TransformFile );

			// Perform the XSLT transformation.  The results are stored in the XmlTextWriter instantiated earlier.
			// If any parameters were passed, include them in the transformation
			if( Parameters != null )
				xslt.Transform( xdoc, Parameters, xmlWriter );
			else
				xslt.Transform( xdoc, null, xmlWriter );

			// Return the transformed data.
			return buffer.ToString();

		}
		catch( System.Exception err )
		{
			throw( err );
		}

	}

	#endregion 

	#region Conversion of IEnumerable (Array, ArrayList, etc) data to Xml

	public static void ArrayToXml( XmlTextWriter writer, IEnumerable data, string rootElement, string rowElement, string dateFormat, string numberFormat )
	{

		writer.WriteStartElement( rootElement );

		foreach( object loop in data )
		{
            
			writer.WriteStartElement( rowElement );
			writer.WriteString( FormatAsXml( loop, dateFormat, numberFormat ) );
			writer.WriteEndElement();

		}

		writer.WriteEndElement();

	}

	public static StringReader ArrayToXml( IEnumerable data, string rootElement, string rowElement, string dateFormat, string numberFormat  )
	{

		// Create a buffer to hold the interim XML code
		StringWriter buffer = new StringWriter();
		XmlTextWriter writer = CreateXmlWriter( buffer );

		// Write the data to the Xml buffer
		ArrayToXml( writer, data, rootElement, rowElement, dateFormat, numberFormat );

		// Return the results as a StringReader object
		StringReader result = new StringReader( buffer.ToString() );
		return result;

	}

	public static StringReader ArrayToXml( IEnumerable data, string rootElement, string rowElement )
	{
		return ArrayToXml( data, rootElement, rowElement, "D", "N" );
	}

	public static StringReader ArrayToXml( IEnumerable data )
	{
		return ArrayToXml( data, "root", "row" );
	}

	#endregion 

	#region Conversion of Collections to Xml 

	public static void CollectionToXml( XmlTextWriter writer, ICollection data, string rootElement, string rowElement, string dateFormat, string numberFormat )
	{

		writer.WriteStartElement( rootElement );

		foreach( object loop in data )
		{
            
			writer.WriteStartElement( rowElement );
			writer.WriteString( FormatAsXml( loop, dateFormat, numberFormat ) );
			writer.WriteEndElement();

		}

		writer.WriteEndElement();

	}

	public static StringReader CollectionToXml( ICollection data, string rootElement, string rowElement, string dateFormat, string numberFormat  )
	{

		// Create a buffer to hold the interim XML code
		StringWriter buffer = new StringWriter();
		XmlTextWriter writer = CreateXmlWriter( buffer );

		// Write the data to the Xml buffer
		CollectionToXml( writer, data, rootElement, rowElement, dateFormat, numberFormat );

		// Return the results as a StringReader object
		StringReader result = new StringReader( buffer.ToString() );
		return result;

	}

	public static StringReader CollectionToXml( ICollection data, string rootElement, string rowElement )
	{
		return CollectionToXml( data, rootElement, rowElement, "D", "N" );
	}

	public static StringReader CollectionToXml( ICollection data )
	{
		return CollectionToXml( data, "root", "row" );
	}

	#endregion 

	#region Conversion of IDictionary (Hashtable, etc) data to Xml

	public static void DictionaryToXml( XmlTextWriter writer, IDictionary data, string rootElement, string dateFormat, string numberFormat )
	{

		writer.WriteStartElement( rootElement );

		foreach( object key in data.Keys )
		{
			writer.WriteStartElement( key.ToString().Replace( " ", "" ) );
			writer.WriteString( XmlHelper.FormatAsXml( data[key], dateFormat, numberFormat ) );
			writer.WriteEndElement();
		}

		writer.WriteEndElement();

	}

	public static StringReader DictionaryToXml( IDictionary data, string rootElement, string dateFormat, string numberFormat  )
	{

		// Create a buffer to hold the interim XML code
		StringWriter buffer = new StringWriter();
		XmlTextWriter writer = CreateXmlWriter( buffer );

		// Write the data to the Xml buffer
		DictionaryToXml( writer, data, rootElement, dateFormat, numberFormat );

		// Return the results as a StringReader object
		StringReader result = new StringReader( buffer.ToString() );
		return result;

	}

	public static StringReader DictionaryToXml( IDictionary data, string rootElement )
	{
		return DictionaryToXml( data, rootElement, "D", "N" );
	}

	public static StringReader DictionaryToXml( IDictionary data )
	{
		return DictionaryToXml( data, "root" );
	}

	#endregion 

}

#endregion
